####################################################################;
# MONTECARLO SIMULATION:  STATIC LOGIT MODEL; 
####################################################################;

# Monte Carlo Simulation of the paper "Panel Quantile Effects Via Distribution Regression"
# Authors: V. Chernozhukov, I. Fernandez-Val, M. Weidner

# Source: Helpman, Melitz, Rubinstein: "Estimating Trade Flows: Trading Partners and Trading Volumes, 2008,
# QJE, 123(2), pp. 441-487;

# Data: Helpman and Melitz web site;

# Std errors assume correct specification;

# 100% increase in distance;

# Source directory in SCC "/project/ivangrp/PDR/";

rm(list=ls());

library(parallel);
library(abind);

load("EmpApp_Est.RData");

#####################
# Extract Arguments #
#####################
#Define default values for variables
null.type <- 2; # 1 for logit DR; 2 for OLS; 3 for tobit;
pwclus <- 0; # Whether the simulation contains pairwise clustering or not;
s.sim  <- 1;
n.boot <- 500;

#Get aruments from the command line
argv <- commandArgs(TRUE);

# Check if the command line is not empty and convert values to numerical values
if (length(argv) > 0){
  null.type <- as.numeric(argv[1]);
  pwclus <- as.numeric(argv[2]);
  s.sim  <- as.numeric(argv[3]);
};

str.null <- ifelse(null.type==1, "", ifelse(null.type==2, "_ols", "_tobit"));
str.pwclus <- ifelse(pwclus,"_pwclus","");

if(null.type!=1){load("Simulation_nullpara.RData")};

source("PDR_Source.R");

####################
# Parallel Setting #
####################

# Calculate the namber of cores
n.cores <- as.numeric(Sys.getenv("NSLOTS"));
if(is.na(n.cores))n.cores<-1;


######################
# Construct new data #
######################
theta.ys <- e.beta[,1:((NCOL(e.beta)-n.K)/2)];
n.obs <- NROW(trade.data);
W.matrix <- model.matrix(model.spec, trade.data);
i.name <- all.vars(model.spec)[n.K+2];
j.name <- all.vars(model.spec)[n.K+3];

set.seed(s.sim);
u <- rnorm(n.obs); # or equivalently qlogis(runif(n.obs));
if(pwclus == 1){
  rho.norm <- 3/4;
  v <- qlogis(pnorm(rho.norm*u + sqrt(1-rho.norm^2)*u[order(trade.data[,j.name],trade.data[,i.name])]));
}else{
  v <- qlogis(pnorm(u));
};

if(null.type == 1){
  trade.data$Ysim <- apply(cbind(W.matrix,v), 1, dgp, theta.0 = theta.ys, ys=ys);
}else if(null.type == 2){
  trade.data$Ysim <- W.matrix%*%theta.ols + v;  
  ys <- ys.ols;
}else if(null.type == 3){
  trade.data$Ysim <- W.matrix%*%theta.tobit + v;
  trade.data$Ysim[trade.data$Ysim < 0] <- 0;
  ys <- ys.tobit;
};


############
# Estimate #
############
model.spec.sim <- update(model.spec, Ysim ~.);

e.beta.sim.l <- mclapply(as.list(ys),PDR.logit, mc.cores=n.cores, model.spec=model.spec.sim, data.local=trade.data);
e.beta.sim   <- do.call(rbind,e.beta.sim.l);
rm(e.beta.sim.l);


theta.ys.sim <- e.beta.sim[,1:((NCOL(e.beta.sim)-n.K)/2)];
e.F.sim.l <- mclapply(1:n.thres, PDR.F.logit, mc.cores=n.cores, model.spec=model.spec.sim, data.local=trade.data, ys=ys, 
                  theta.ys=theta.ys.sim, t.log2=t.log2, t.d=t.d); # t.log2/t.d is the index of log continuos/binary treatment(the place in model.spec);
e.F.sim <- do.call(rbind, e.F.sim.l); # n.thres*8;
rm(e.F.sim.l);


################
# Construct CB #
################
library(pryr);
print(mem_used());
phi.all.sim.l <- mclapply(1:n.thres, phi.logit, mc.cores=n.cores, model.spec=model.spec.sim, 
                          data.local=trade.data, ys=ys, theta.ys=theta.ys.sim, t.log2=t.log2, t.d=t.d);
phi.all.sim <- do.call(abind, list(phi.all.sim.l,along=3)); # n.obs*(n.K+n.I+n.J-1 + 4)*n.thres;
rm(phi.all.sim.l);

var.sim <- colMeans(phi.all.sim^2)/n.obs;



# trade.data(i.e. phi.all) is ordered by id then by jd, now obtain the reversed ordered phi # 
phi.all.sim.reversed <- phi.all.sim[order(trade.data[,j.name],trade.data[,i.name]),,];
var.sim.pwclus <- colMeans((phi.all.sim+phi.all.sim.reversed)*phi.all.sim)/n.obs;
rm(phi.all.sim.reversed);



boot.sim.l <- mclapply(1:n.boot, boot.func, mc.cores=ceiling(n.cores/2), phi=phi.all.sim, variance=var.sim);
boot.sim <- do.call(abind,list(boot.sim.l,along=3)); # (n.thres+1)*(n.K+n.I+n.J-1 + 4)*n.boot;

boot.sim.pwclus.l <- mclapply(1:n.boot, boot.func.pwclus, mc.cores=ceiling(n.cores/2), model.spec=model.spec.sim, data.local=trade.data, 
                              phi=phi.all.sim, variance=var.sim.pwclus);
boot.sim.pwclus <- do.call(abind,list(boot.sim.pwclus.l,along=3));


t.sim <- boot.sim[n.thres+1, ,];
t.sim.pwclus <- boot.sim.pwclus[n.thres+1, ,];

alpha <- 0.05;
crt.sim <- apply(t.sim, 1, quantile, 1-alpha, na.rm=T);
crt.sim.pwclus <- apply(t.sim.pwclus, 1, quantile, 1-alpha, na.rm=T);



F.name <- c(paste("F.",X.name[t.log2],".0",sep=""), paste("F.",X.name[t.log2],".1",sep=""),
            paste("F.",X.name[t.d],".0",sep=""), paste("F.",X.name[t.d],".1",sep=""));
sim <- rbind(cbind(theta.ys.sim,e.F.sim[,F.name]),
             cbind(e.beta.sim[,(NCOL(e.beta.sim)-n.K+1):NCOL(e.beta.sim)], 
                   e.beta.sim[,(n.K+1):((NCOL(e.beta.sim)-n.K)/2)],e.F.sim[,paste(F.name,".bc",sep="")]),
             t(var.sim),
             t(var.sim.pwclus),
             crt.sim,
             crt.sim.pwclus); # (4*n.thres+2)*(n.K+n.I+n.J-1+4);


saveRDS(sim, paste(getwd(),"/Simulations/Sim",str.null,str.pwclus,s.sim,".rds",sep=""));