####################################################################;
# MONTECARLO SIMULATION:  STATIC LOGIT MODEL; 
####################################################################;

# Monte Carlo Simulation of the paper "Panel Quantile Effects Via Distribution Regression"
# Authors: V. Chernozhukov, I. Fernandez-Val, M. Weidner

# Source: Helpman, Melitz, Rubinstein: "Estimating Trade Flows: Trading Partners and Trading Volumes, 2008,
# QJE, 123(2), pp. 441-487;

# Data: Helpman and Melitz web site;

# Std errors assume correct specification;

# 100% increase in distance;

# Source directory in SCC "/project/ivangrp/PDR/";

rm(list=ls());

library(abind);

load("EmpApp_Est.RData");
source("PDR_Source.R");

null.type <- 3; # 1 for logit DR; 2 for OLS; 3 for tobit;
pwclus <- 1; # Whether the simulation is pairwise clustering or not;

str.null <- ifelse(null.type==1, "", ifelse(null.type==2, "_ols", "_tobit"));
str.pwclus <- ifelse(pwclus,"_pwclus","");

if(null.type!=1){load("Simulation_nullpara.RData")};

#################################
## Load the simulation results ##
#################################
n.sim <- 100;

sim.result <- readRDS(paste(getwd(),"/Simulations/Sim",str.null,str.pwclus,"1.rds",sep=""));
for (i in 2:n.sim){
  sim.result <- abind(sim.result, readRDS(paste(getwd(),"/Simulations/Sim",str.null,str.pwclus,i,".rds",sep="")), along=3);
}; # (3*n.thres+2)*(n.K+n.I+n.J-1 + 4)*n.sim;

q.cut  <- c(1:82); # 85% - 1:62; 90% - 1:72;
q.plot <- c(1:82); # 95% - 1:82;

##################################
## Construct Simulation Measure ##
##################################
F.name <- c(paste("F.",X.name[t.log2],".0",sep=""), paste("F.",X.name[t.log2],".1",sep=""),
            paste("F.",X.name[t.d],".0",sep=""), paste("F.",X.name[t.d],".1",sep=""));
if(null.type == 1){ 
  theta.0 <- cbind(e.beta[,1:((NCOL(e.beta)-n.K)/2)],e.F[,F.name]);
}else if(null.type == 2){
  theta.0 <- cbind(-t(replicate(n.thres,theta.ols)), e.F.ols);
}else if(null.type == 3){
  theta.0 <- cbind(-t(replicate(n.thres,theta.tobit)), e.F.tobit);
};

sim.theta    <- sim.result[1:n.thres,,];
sim.theta.bc <- sim.result[(n.thres+1):(2*n.thres),,];

bias    <- rowMeans(sim.theta,na.rm=T,dims=2) - theta.0; # n.thres*(n.K+n.I+n.J-1 + 4);
bias.bc <- rowMeans(sim.theta.bc,na.rm=T,dims=2) - theta.0;

sd    <- apply(sim.theta,c(1,2),sd,na.rm=T);
sd.bc <- apply(sim.theta.bc,c(1,2),sd,na.rm=T);

rmse    <- sqrt(rowMeans((sim.theta-replicate(n.sim,theta.0))^2,na.rm=T,dims=2));
rmse.bc <- sqrt(rowMeans((sim.theta.bc-replicate(n.sim,theta.0))^2,na.rm=T,dims=2));


# For Confidence Bands #

se      <- rowMeans(sqrt(sim.result[(2*n.thres+1):(3*n.thres),,]),na.rm=T,dims=2);
sesd    <- se/sd;
sesd.bc <- se/sd.bc;

crt.sim <- sim.result[4*n.thres+1,,];
crt     <- rowMeans(crt.sim, na.rm=T); # (n.K+n.I+n.J-1+4)*1;

width.sim <- 2*sqrt(sim.result[(2*n.thres+1):(3*n.thres),,])*aperm(replicate(n.thres,crt.sim),c(3,1,2));
width     <- rowMeans(width.sim,na.rm=T,dims=2); # n.thres*(n.K+n.I+n.J-1 + 4);

ind.sim    <- sim.theta+width.sim/2>=replicate(n.sim,theta.0)&sim.theta-width.sim/2<=replicate(n.sim,theta.0);
ind.sim.bc <- sim.theta.bc+width.sim/2>=replicate(n.sim,theta.0)&sim.theta.bc-width.sim/2<=replicate(n.sim,theta.0);

coverage    <- rowMeans(apply(ind.sim[q.cut,,],c(2,3),min,na.rm=T),na.rm=T); # (n.K+n.I+n.J-1 + 4)*1;
coverage.bc <- rowMeans(apply(ind.sim.bc[q.cut,,],c(2,3),min,na.rm=T),na.rm=T);


# Pointwise CI:
alpha <- 0.05;
width.sim.pw <- 2*qnorm(1-alpha/2)*sqrt(sim.result[(2*n.thres+1):(3*n.thres),,]);
width.pw     <- rowMeans(width.sim.pw,na.rm=T,dims=2); # n.thres*(n.K+n.I+n.J-1 + 4);

ind.pw    <- sim.theta+width.sim.pw/2>=replicate(n.sim,theta.0)&sim.theta-width.sim.pw/2<=replicate(n.sim,theta.0);
ind.pw.bc <- sim.theta.bc+width.sim.pw/2>=replicate(n.sim,theta.0)&sim.theta.bc-width.sim.pw/2<=replicate(n.sim,theta.0);

coverage.pw    <- rowMeans(apply(ind.pw[q.cut,,],c(2,3),min,na.rm=T),na.rm=T); # (n.K+n.I+n.J-1 + 4)*1;
coverage.pw.bc <- rowMeans(apply(ind.pw.bc[q.cut,,],c(2,3),min,na.rm=T),na.rm=T);


#######################
# Pairwise Clustering #
#######################

se.pwclus      <- rowMeans(sqrt(sim.result[(3*n.thres+1):(4*n.thres),,]),na.rm=T,dims=2);
sesd.pwclus    <- se.pwclus/sd;
sesd.bc.pwclus <- se.pwclus/sd.bc;
  
crt.sim.pwclus <- sim.result[4*n.thres+2,,];
crt.pwclus     <- rowMeans(crt.sim.pwclus,na.rm=T);
  
width.sim.pwclus <- 2*sqrt(sim.result[(3*n.thres+1):(4*n.thres),,])*aperm(replicate(n.thres,crt.sim.pwclus),c(3,1,2));
width.pwclus     <- rowMeans(width.sim.pwclus,na.rm=T,dims=2);
  
ind.sim.pwclus    <- sim.theta+width.sim.pwclus/2>=replicate(n.sim,theta.0)&sim.theta-width.sim.pwclus/2<=replicate(n.sim,theta.0);
ind.sim.bc.pwclus <- sim.theta.bc+width.sim.pwclus/2>=replicate(n.sim,theta.0)&sim.theta.bc-width.sim.pwclus/2<=replicate(n.sim,theta.0);
  
coverage.pwclus    <- rowMeans(apply(ind.sim.pwclus[q.cut,,],c(2,3),min,na.rm=T),na.rm=T);
coverage.bc.pwclus <- rowMeans(apply(ind.sim.bc.pwclus[q.cut,,],c(2,3),min,na.rm=T),na.rm=T);
  
# Pointwise CI:
width.sim.pw.pwclus <- 2*qnorm(1-alpha/2)*sqrt(sim.result[(3*n.thres+1):(4*n.thres),,]);
width.pw.pwclus     <- rowMeans(width.sim.pw.pwclus,na.rm=T,dims=2);
  
ind.pw.pwclus    <- sim.theta+width.sim.pw.pwclus/2>=replicate(n.sim,theta.0)&sim.theta-width.sim.pw.pwclus/2<=replicate(n.sim,theta.0);
ind.pw.bc.pwclus <- sim.theta.bc+width.sim.pw.pwclus/2>=replicate(n.sim,theta.0)&sim.theta.bc-width.sim.pw.pwclus/2<=replicate(n.sim,theta.0);
 
coverage.pw.pwclus    <- rowMeans(apply(ind.pw.pwclus[q.cut,,],c(2,3),min,na.rm=T),na.rm=T);
coverage.pw.bc.pwclus <- rowMeans(apply(ind.pw.bc.pwclus[q.cut,,],c(2,3),min,na.rm=T),na.rm=T);




###############
# Print Table #
###############
print.para <- c("ldist","legal","F.ldist.0","F.ldist.1","F.legal.0","F.legal.1");
table <- rbind(colMeans(width[q.cut,])[print.para],
               crt[print.para],
               coverage[print.para]*100,
               coverage.pw[print.para]*100,
               colMeans(sesd[q.cut,])[print.para],
               
               colMeans(width[q.cut,])[print.para],
               crt[print.para],
               coverage.bc[print.para]*100,
               coverage.pw.bc[print.para]*100,
               colMeans(sesd.bc[q.cut,])[print.para]);
colnames(table) <- print.para;
entries <- c("Average Length",
             "Average Critical Value",
             "Coverage uniform band(%)",
             "Coverage pointwise band(%)",
             "Average SE/SD");
rownames(table) <- c(entries, paste(entries," BC",paste=""));


table.clus <- rbind(colMeans(width.pwclus[q.cut,])[print.para],
                    crt.pwclus[print.para],
                    coverage.pwclus[print.para]*100,
                    coverage.pw.pwclus[print.para]*100,
                    colMeans(sesd.pwclus[q.cut,])[print.para],
                      
                    colMeans(width.pwclus[q.cut,])[print.para],
                    crt.pwclus[print.para],
                    coverage.bc.pwclus[print.para]*100,
                    coverage.pw.bc.pwclus[print.para]*100,
                    colMeans(sesd.bc.pwclus[q.cut,])[print.para]);
colnames(table) <- print.para;
rownames(table.clus) <- c(entries, paste(entries," BC",paste=""));



save.image(paste("Results_Sim",str.null,str.pwclus,".RData",sep=""));

library(xtable);
options(xtable.floating=FALSE);
options(xtable.timestamp="");

xtable(table);
xtable(table.clus);


###############
# Plot Graphs #
###############

# Coefficient #

pdf(paste("Sim",str.null,str.pwclus,"_coef.pdf",sep=""));
  para.name <- c("ldist","legal");
  if(null.type==1){
    ylimlist <- rbind(c(-5,17),c(-11,41));
  }else if(null.type==2){
    ylimlist <- rbind(c(-1,38),c(-1,53));
  }else if(null.type==3){
    ylimlist <- rbind(c(0,10),c(0,68));
  };

  for(i in 1:length(para.name)){
    matplot(taus[q.plot],
            cbind(bias[q.plot,para.name[i]]/theta.0[q.plot,para.name[i]]*100,
                  bias.bc[q.plot,para.name[i]]/theta.0[q.plot,para.name[i]]*100),
            col=c("black","red"),lty=1,type="l",ylim=ylimlist[i,],
            xlab="Quantile of Trade",ylab="% of True Parameter Value",main=paste("Bias of ",para.name[i],sep=""));
    abline(h=0,lty=2);
    legend("topleft",legend=c("Uncorrected","Bias Corrected"),col=c("black","red"),lty=1);
  
    matplot(taus[q.plot],
            abs(cbind(sd[q.plot,para.name[i]]/theta.0[q.plot,para.name[i]]*100,
                      sd.bc[q.plot,para.name[i]]/theta.0[q.plot,para.name[i]]*100)),
            col=c("black","red"),lty=1,type="l",ylim=ylimlist[i,],
            xlab="Quantile of Trade",ylab="% of True Parameter Value",main=paste("SD of ",para.name[i],sep=""));
    abline(h=0,lty=2);
    legend("topleft",legend=c("Uncorrected","Bias Corrected"),col=c("black","red"),lty=1);
    
    matplot(taus[q.plot],
            abs(cbind(rmse[q.plot,para.name[i]]/theta.0[q.plot,para.name[i]]*100,
                      rmse.bc[q.plot,para.name[i]]/theta.0[q.plot,para.name[i]]*100)),
            col=c("black","red"),lty=1,type="l",ylim=ylimlist[i,],
            xlab="Quantile of Trade",ylab="% of True Parameter Value",main=paste("RMSE of ",para.name[i],sep=""));
    abline(h=0,lty=2);
    legend("topleft",legend=c("Uncorrected","Bias Corrected"),col=c("black","red"),lty=1);
    
    matplot(taus[q.plot],
            cbind(sesd[q.plot,para.name[i]],
                  sesd.bc[q.plot,para.name[i]]),
            col=c("black","red"),lty=1,type="l",ylim=c(0.7,1.2),
            xlab="Quantile of Trade",ylab="% of True Parameter Value",main=paste("SE/SD of ",para.name[i],sep=""));
    abline(h=1,lty=2);
    legend("topleft",legend=c("Uncorrected SD","Bias Corrected SD"),col=c("black","red"),lty=1);
    
    matplot(taus[q.plot],
            cbind(sesd.pwclus[q.plot,para.name[i]],
                  sesd.bc.pwclus[q.plot,para.name[i]]),
            col=c("black","red"),lty=1,type="l",ylim=c(0.7,1.2),
            xlab="Quantile of Trade",ylab="% of True Parameter Value",main=paste("Pairwise Clustered SE/SD of ",para.name[i],sep=""));
    abline(h=1,lty=2);
    legend("topleft",legend=c("Uncorrected SD","Bias Corrected SD"),col=c("black","red"),lty=1);
    
  };

dev.off();



# Distribution Function #

pdf(paste("Sim",str.null,str.pwclus,"_df.pdf",sep=""));
  para.name <- c("ldist","legal");
  #ylimlist <- abind(cbind(c(0,0.6),c(-0.2,0.9)),cbind(c(0,0.6),c(-0.2,0.9)),along=3);
  if(null.type==1){
    ylimlist <- rbind(c(0,1),c(-0.2,1.5));
    ylimsesd <- c(0.5,1.25);
  }else if(null.type==2){
    ylimlist <- rbind(c(-0.1,6),c(-0.1,7));
    ylimsesd <- c(0.1,1.2);
  }else if(null.type==3){
    ylimlist <- rbind(c(0,0.5),c(-0.1,0.8));
    ylimsesd <- c(0.1,1.2);
  };
  
  for(i in 1:length(para.name)){
    for(treated in c(0,1)){
      matplot(taus[q.plot],
              cbind(bias[q.plot,paste("F.",para.name[i],".",treated,sep="")]/theta.0[q.plot,paste("F.",para.name[i],".",treated,sep="")]*100,
                    bias.bc[q.plot,paste("F.",para.name[i],".",treated,sep="")]/theta.0[q.plot,paste("F.",para.name[i],".",treated,sep="")]*100),
              col=c("black","red"),lty=1,type="l",ylim=ylimlist[i,],
              xlab="Quantile of Trade",ylab="% of True Parameter Value",
              main=paste("Bias of Counterfactual Distribution -- ",para.name[i],", k=",treated,sep=""));
      abline(h=0,lty=2);
      legend("topleft",legend=c("Uncorrected","Bias Corrected"),col=c("black","red"),lty=1);
      
      matplot(taus[q.plot],
              cbind(sd[q.plot,paste("F.",para.name[i],".",treated,sep="")]/theta.0[q.plot,paste("F.",para.name[i],".",treated,sep="")]*100,
                    sd.bc[q.plot,paste("F.",para.name[i],".",treated,sep="")]/theta.0[q.plot,paste("F.",para.name[i],".",treated,sep="")]*100),
              col=c("black","red"),lty=1,type="l",ylim=ylimlist[i,],
              xlab="Quantile of Trade",ylab="% of True Parameter Value",
              main=paste("SD of Counterfactual Distribution -- ",para.name[i],", k=",treated,sep=""));
      abline(h=0,lty=2);
      legend("topleft",legend=c("Uncorrected","Bias Corrected"),col=c("black","red"),lty=1);
      
      matplot(taus[q.plot],
              cbind(rmse[q.plot,paste("F.",para.name[i],".",treated,sep="")]/theta.0[q.plot,paste("F.",para.name[i],".",treated,sep="")]*100,
                    rmse.bc[q.plot,paste("F.",para.name[i],".",treated,sep="")]/theta.0[q.plot,paste("F.",para.name[i],".",treated,sep="")]*100),
              col=c("black","red"),lty=1,type="l",ylim=ylimlist[i,],
              xlab="Quantile of Trade",ylab="% of True Parameter Value",
              main=paste("RMSE of Counterfactual Distribution -- ",para.name[i],", k=",treated,sep=""));
      abline(h=0,lty=2);
      legend("topleft",legend=c("Uncorrected","Bias Corrected"),col=c("black","red"),lty=1);
      
      
      
      matplot(taus[q.plot],
              cbind(sesd[q.plot,paste("F.",para.name[i],".",treated,sep="")],
                    sesd.bc[q.plot,paste("F.",para.name[i],".",treated,sep="")]),
              col=c("black","red"),lty=1,type="l",ylim=ylimsesd,
              xlab="Quantile of Trade",ylab="% of True Parameter Value",
              main=paste("SE/SD of Counterfactual Distribution -- ",para.name[i],", k=",treated,sep=""));
      abline(h=1,lty=2);
      legend("topleft",legend=c("Uncorrected SD","Bias Corrected SD"),col=c("black","red"),lty=1);
      
      matplot(taus[q.plot],
              cbind(sesd.pwclus[q.plot,paste("F.",para.name[i],".",treated,sep="")],
                    sesd.bc.pwclus[q.plot,paste("F.",para.name[i],".",treated,sep="")]),
              col=c("black","red"),lty=1,type="l",ylim=ylimsesd,
              xlab="Quantile of Trade",ylab="% of True Parameter Value",
              main=paste("Pairwise Clustered SE/SD of Counterfactual Distribution -- ",para.name[i],", k=",treated,sep=""));
      abline(h=1,lty=2);
      legend("topleft",legend=c("Uncorrected SD","Bias Corrected SD"),col=c("black","red"),lty=1);
      
    };
};

dev.off();